#! /home/nmoreau/.ismpy/bin/python3
# -*- utf-8 -*-
from threading import Thread, Event
import argparse
import sys
import os.path
import time
import socket
from contextlib import closing
import cli_extraction as cli
import loader
import socket
import threading

from server.lib import uws
from server.lib import server
from server.lib import config



class JobUws(Thread):
    def __init__(self):
        Thread.__init__(self)

    def run(self):
        uws.run()


class Server(Thread):
    def __init__(self):
        Thread.__init__(self)

    def run(self):
        server.listen()            

class CLIExecution(Thread):
    def __init__(self):
        Thread.__init__(self)
        self.can_run = Event()

    def run(self):
        print("Starting extraction process")
        tries = 0
        connect = False

        while tries < 1000 and connect is False:
            try:
                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            except socket.error as e:
                print(e)
                
            try:
                s.connect(("127.0.0.1", config.SERVER_PORT))
                connect = True
                cli.process([args['file']], 
                             args['template'], 
                             args["output"], 
                             args['yaml'])
                print("\n\nProcess completed.")
                return
            except Exception as e:
                pass
            tries = tries + 1  
        print("Error : could not connect to local server.")    
        sys.exit(0)
        

    def resume(self):
        self.can_run.set()



def startserver():
    """
    start server and job execution in separated threads
    """
    threaduws = JobUws()
    threaduws.start()
    threadserver = Server()
    threadserver.start()


def parseargs():
    description = "Run extractor service"
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument("-s", "--start",
                        help="run server and launch app", action='store_true')
    parser.add_argument("--server",
                        help="run server only", action='store_true')
    parser.add_argument("-f", "--file",
                        help="path towards the HDF5 file")
    parser.add_argument("-t", "--template",
                        help="path towards the ASCII file with the list \
                             of quantities to extract")
    parser.add_argument("-o", "--output",
                        help="path and name of the ASCII output file \
                              to produce.")
    parser.add_argument("-y", "--yaml",
                        help="optional: path towards a yaml file \
                              to produce plot")    

    args = vars(parser.parse_args())
    return args

def checkargs(args):
    if args['file'] is not None:
        if os.path.isfile(os.path.join(config.DATA_DIRECTORY, args['file'])) is False and \
           os.path.isfile(args['file']) is False :
            raise Exception(f"File {args['file']} not found," +
                              "please check that it is available in " +
                              "the material/data directory.")
    else:
        if args["start"] is False and args["server"] is False:
            raise Exception("Not enough parameters to run the application")



def actions(args):
    if len(args) == 1:
        print("Nothing to do, exiting program")
        sys.exit()
    if args['server'] is True:
        threading.Timer(config.PAGE_OPENING_TEMPO, loader.display_app_uri).start()
    elif args['start'] is True:
        # start server, wait and display data loading gui
        threading.Timer(config.PAGE_OPENING_TEMPO, loader.goto_form).start()
    elif args['start'] is False and \
         args['file'] is not None and \
         args['template'] is None and \
         args['output'] is None:
        # start server, wait and display file content
        threading.Timer(config.PAGE_OPENING_TEMPO, loader.goto_file, [args['file']]).start()
    else:
        # CLI interface
        if args['file'] is None:
            print("Error : the path to hdf5 file is missing.")
            sys.exit()

        if args['template'] is None:
            print("Error : the list of quantities to extract is missing.")
            sys.exit()

        if args['output'] is None:
            print("Error : the path to extracted data is missing.")
            sys.exit()
        cli = CLIExecution()
        cli.start()
        
def is_server_running():
    """
        return True if the server port is already in use
    """
    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
        if sock.connect_ex(("127.0.0.1", config.SERVER_PORT)) == 0:
            return True
        else:
            return False


if __name__ == "__main__":    
    args = parseargs()
    try : 
        checkargs(args)
    except Exception as e:
        print(e)
        sys.exit()
    # if server is already running, use existing instance
    if is_server_running() is False:        
        try:
            startserver()            
        except OSError as e:
            print(e)
    else:
        print(f"Warning : Port {config.SERVER_PORT} is already used, an instance of the server is probably running.")

    actions(args)
        
        

